#lang at-exp racket/gui 
(require infix plot)

(define r 30)
(define angle 0)

(define (cycloid t) @${vector[r*(t - sin[t]),  r*(1-cos[t])]})

(define (line t)
  (let ([x @${r*(t - sin[t])}]
        [y @${r*(1 - cos[t])}]
        [x0 (* r angle)]
        [y0 r])
    (lines (list (vector x0 y0) (vector x y))
           #:width 2
           #:color "blue")))

(define (end-points t)
  (let ([x @${r*(t - sin[t])}]
        [y @${r*(1 - cos[t])}]
        [x0 (* r angle)]
        [y0 r])
    (points (list (vector x0 y0) (vector x y))
            #:size 10
            #:sym 'fullcircle1
            #:color "red"
            #:fill-color "red")))

(define (circle t) @${vector[r*angle + r*sin[t], r + r*cos[t]]})

(plot-decorations? #f)

(define (cycloid-plot dc)
  (plot/dc (list
            (axes)
            (parametric circle 0 (* 2 pi) #:color "green")
            (parametric cycloid 0 angle #:color "red")
            (line angle)
            (end-points angle))
           dc 10 25 300 150
           #:x-min 0
           #:x-max (* r 2 pi)
           #:y-min 0
           #:y-max (* r pi))) 

(define main-frame
  (new frame%
       [label "The Plot Thickens"]
       [width 350]
       [height 250]))

(define canvas
  (new canvas% [parent main-frame]
       [paint-callback
        (lambda (canvas dc)
          (send dc set-smoothing 'smoothed)
          (cycloid-plot dc)
          (send dc set-text-foreground "blue")
          (send dc draw-text "Animated Cycloid" 10 180))]))

(define timer
  (new timer%
       [notify-callback
        (lambda ()
          (set! angle (+ 0.1 angle))
          (when (> angle (* 2 pi)) (set! angle 0))
          (send canvas refresh-now))]))
       
(send main-frame show #t)

(send timer start 10)