#lang racket

;change path as needed

(define (file-info sym period)
  (let ([port (open-input-file
               (build-path "StockHistory" period (string-append sym ".csv")))]
        [recs 0]
        [max-size 0])
    (let loop ()
      (let ([rec (read-line port)])
        (unless (eof-object? rec)
          (let ([len (string-length rec)])
            (set! recs (add1 recs))
            (unless (<= len max-size)
              (set! max-size len)))
          (loop))))
    (close-input-port port)
    (values recs max-size)))

(define (csv->db sym period)
  (let*-values ([(recs max-size) (file-info sym period)]
                [(in-port) (open-input-file
                            (build-path "StockHistory" period (string-append sym ".csv")))]
                [(out-port) (open-output-file
                             (build-path "StockHistory" period (string-append sym ".db"))
                             #:exists 'truncate)]
                [(slot-size) (+ 10 max-size)])
    (file-position out-port (* recs slot-size))
    (display (make-string slot-size #\space) out-port)
    (file-position out-port 0)
    (display recs out-port)
    (display "," out-port)
    (displayln slot-size out-port)
    (read-line in-port) ; read past header
    (for ([ i (in-range 1 recs)])
      (let ([rec (read-line in-port)]
            [pos (* i slot-size)])
        (file-position out-port pos)
        (displayln rec out-port)))
    (close-input-port in-port)
    (close-output-port out-port)
    ))

(define (bin-search sym period date)
  (let* ([port (open-input-file
                (build-path "StockHistory" period (string-append sym ".db")))]
         [info (string-split (read-line port) ",")]
         [recs (string->number (first info))]
         [slot-size (string->number (second info))]
         [min 1]
         [max recs])
    
    (define (get-date rec)
      (substring rec 0 10))

    (define (get-rec i)
      (file-position port (* slot-size i))
      (read-line port))
    
    (let loop ()
      (if (> min max)
          (begin
            (close-input-port port)
            #f)
          (let* ([i (floor (/ (+ min max) 2))]
                 [rec (get-rec i)]
                 [d (get-date rec)])
            (cond [(string<? d date)
                   (set! min (add1 i))
                   (loop)]
                  [(string>? d date)
                   (set! max (sub1 i))
                   (loop)]
                  [else
                   (close-input-port port)
                   rec]))))))
